@@ -20,7 +20,7 @@ class Command:
     def __init__(self):
         self.help = "Create a new app in ./apps"
 
-    def handle(self, args: List[str]) -> None:
+    async def handle(self, args: List[str]) -> None:
         if not args:
             print("Usage: neutronapi startapp <app_name>")
             return
@@ -47,4 +47,3 @@ class Command:
             f.write(MODELS_TEMPLATE.format(app_name=app_name))
 
         print(f"✓ App '{app_name}' created at apps/{app_name}")
-
